@@ -56,10 +56,10 @@ def symbol_map_from_instance(instance):
    return resulting_map
 
 #
-# a symbol map is a mechanism for tracking assigned labels (e.g., for use when writing 
-# problem files for input to an optimizer) for objects in a particular problem instance.
+# a symbol map is a mechanism for tracking assigned labels (e.g., for
+# use when writing problem files for input to an optimizer) for objects
+# in a particular problem instance.
 #
-
 class SymbolMap(object):
 
     class UnknownSymbol:
@@ -67,10 +67,11 @@ class SymbolMap(object):
 
     def __init__(self, instance):
 
-        # conceptually, a symbol map must be associated with an instance - for example,
-        # the byObject map creates associations between object ids and their symbols
-        # => the ids are tied to a specific instance. however, we don't actually do
-        # anything with the instance within this class quite yet.
+        # conceptually, a symbol map must be associated with an instance
+        # - for example, the byObject map creates associations between
+        # object ids and their symbols => the ids are tied to a specific
+        # instance. however, we don't actually do anything with the
+        # instance within this class quite yet.
         self.instance = instance
         
         # maps object id()s to their assigned symbol.
@@ -93,21 +94,39 @@ class SymbolMap(object):
     #
     def getByObjectDictionary(self):
         return self.byObject
- 
 
     #
-    # invoked when the caller guarantees that a name conflict will not arise. use with care!
+    # invoked when the caller guarantees that a name conflict will not
+    # arise. use with care!
     #
     def createSymbol(self, obj, labeler, *args):
 
         # the following test is slightly faster than always calling *args.
         if args:
-           ans = labeler(obj, *args)
+            ans = labeler(obj, *args)
         else:
-           ans = labeler(obj)
+            ans = labeler(obj)
         self.byObject[id(obj)] = ans
         self.bySymbol[ans] = obj
         return ans
+
+    #
+    # same as above, but with a list (more generally, an iterable) of objs.
+    #
+    def createSymbols(self, objs, labeler, *args):
+
+       byObjectDictionary = self.byObject
+       bySymbolDictionary = self.bySymbol
+
+       for obj in objs:
+           # the following test is slightly faster than always calling *args.
+           if args:
+               ans = labeler(obj, *args)
+           else:
+               ans = labeler(obj)
+           # TBD - ADD BULK ADD TO DICTIONARY - VIA UPDATE?
+           byObjectDictionary[id(obj)] = ans
+           bySymbolDictionary[ans] = obj
   
     #
     # same as above, but with full error checking for duplicates / collisions.
@@ -116,19 +135,21 @@ class SymbolMap(object):
 
         obj_id = id(obj)
         if obj_id in self.byObject:
-           return self.byObject[obj_id]
+            return self.byObject[obj_id]
 
         # the following test is slightly faster than always calling *args
         if args:
-           ans = labeler(obj, *args)
+            ans = labeler(obj, *args)
         else:
-           ans = labeler(obj)
+            ans = labeler(obj)
            
-        if self.bySymbol.setdefault(ans, obj) is not obj:
-           raise RuntimeError(
-              "Duplicate symbol '%s' already associated with "
-              "component '%s' (conflicting component: '%s')"
-              % (ans, self.bySymbol[ans].name, obj.name) )
+        if ans in self.bySymbol:
+            if self.bySymbol[ans] is not obj:
+                raise RuntimeError(
+                    "Duplicate symbol '%s' already associated with "
+                    "component '%s' (conflicting component: '%s')"
+                    % (ans, self.bySymbol[ans].name, obj.name) )
+        self.bySymbol[ans] = obj
         self.byObject[obj_id] = ans
         return ans
 
@@ -138,18 +159,22 @@ class SymbolMap(object):
         #    raise RuntimeError(
         #        "Cannot alias object '%s': object not in SymbolMap."
         #        % ( name, ))
-        if self.aliases.setdefault(name, obj) is not obj:
-            raise RuntimeError(
-                "Duplicate alias '%s' already associated with "
-                "component '%s' (conflicting component: '%s')"
-                % (name, self.aliases[name].name, obj.name) )
+        #if self.aliases.setdefault(name, obj) is not obj:
+        if name in self.aliases:
+            if self.aliases[name] is not obj:
+                raise RuntimeError(
+                    "Duplicate alias '%s' already associated with "
+                    "component '%s' (conflicting component: '%s')"
+                    % (name, self.aliases[name].name, obj.name) )
+        self.aliases[name] = obj
 
     def getObject(self, symbol):
-
-        ans = self.bySymbol.get(symbol, SymbolMap.UnknownSymbol)
-        if ans is SymbolMap.UnknownSymbol:
-            ans = self.aliases.get(symbol, SymbolMap.UnknownSymbol)
-        return ans
+        if symbol in self.bySymbol:
+            return self.bySymbol[symbol]
+        elif symbol in self.aliases:
+            return self.aliases[symbol]
+        else:
+            return SymbolMap.UnknownSymbol
             
     def getEquivalentObject(self, symbol, instance):
 
@@ -182,8 +207,3 @@ class SymbolMap(object):
         if idx is not None:
             obj = obj[idx]
         return obj
-                
-        
-        
-
-        
